#pragma once

#include <cgv/data/ref_ptr.h>
#include <cgv/base/lib_begin.h>
#include <iostream>

#ifdef WIN32
#pragma warning(disable:4275)
#pragma warning(disable:4231)
#endif

namespace cgv {
	namespace base {

class CGV_API base;
class CGV_API named;
class CGV_API node;
class CGV_API group;

/// ref counted pointer to a node
typedef data::ref_ptr<base,true> base_ptr;

template <class T> struct cast_helper { static T* cast(base* b) { return dynamic_cast<T*>(b); } };

/** base class for all gui types */
class CGV_API base : public data::ref_counted
{
public:
	/// make destructor virtual
	virtual ~base();
	/// overload to return the type name of this object
	virtual const char* get_type_name() const = 0;
	/// overload to show the content of this object
	virtual void stream_stats(std::ostream&);
	/// cast upward to named
	virtual named* get_named();
	/// cast upward to node
	virtual node* get_node();
	/// cast upward to group
	virtual group* get_group();
	/// cast to arbitrary class, but use the casts to named, node and group from the interface
	template <class T>
	T* cast() {
		return cast_helper<T>::cast(this);
	}
};

template <> struct cast_helper<named>{ static named* cast(base* b) { return b->get_named(); } };
template <> struct cast_helper<node>{ static node* cast(base* b) { return b->get_node(); } };
template <> struct cast_helper<group>{ static group* cast(base* b) { return b->get_group(); } };

#ifdef WIN32
CGV_TEMPLATE template class CGV_API data::ref_ptr<base>;
#endif

	}
}

#include <cgv/config/lib_end.h>