#pragma once

#include <vector>
#include <cgv/base/node.h>

#include <cgv/base/lib_begin.h>

namespace cgv {
	namespace base {

class CGV_API group;

/// ref counted pointer to a node
typedef data::ref_ptr<group,true> group_ptr;

/** The group class is a node with children. */
class CGV_API group : public node
{
protected:
	/// store a list of children
	std::vector<base_ptr> children;
	/// check if the base class is a node and set the parent of the node
	void link(base_ptr b);
	/// check if the base class is a node and clear the parent of the node
	void unlink(base_ptr b);
public:
	/// construct from name
	group(const char* name);
	/// return the number of children
	unsigned int get_nr_children() const;
	/// return the i-th child
	base_ptr get_child(unsigned int i) const;
	/// append child and return index of appended child
	unsigned int append_child(base_ptr child);
	/// remove all elements of the vector that point to child, return the number of removed children
	unsigned int remove_child(base_ptr child);
	/// remove all children
	void remove_all_children();
	/// insert a child at the given position
	void insert_child(unsigned int i, base_ptr child);
	/// cast upward to group
	group* get_group();
	/// overload to return the type name of this object
	const char* get_type_name() const;
};

#ifdef WIN32
CGV_TEMPLATE template class CGV_API data::ref_ptr<group>;
CGV_TEMPLATE template class CGV_API std::vector<base_ptr>;
#endif

	}
}

#include <cgv/config/lib_end.h>