#include <cgv/base/register.h>
#include <vector>
#include <cgv/media/text/convert.h>

namespace cgv {
	namespace base {

struct factory_entry
{
	abst_factory* factory;
	const char* item_text;
	char shortcut;
	factory_entry(abst_factory* _factory=0, const char* _item_text=0, char _shortcut=0):
	factory(_factory), item_text(_item_text), shortcut(_shortcut) {}
};


bool registration_enabled = true;

unsigned int nr_factories_before_disable = 0;

unsigned int nr_objects_before_disable = 0;

std::vector<abst_registration_listener*> listeners;

std::vector<factory_entry> factories;

std::vector<base_ptr> objects;

abst_factory::abst_factory(bool _singelton) : is_singelton(_singelton) 
{
}

/// if registration is disable, all registration events are stored and sent at the momement when registration is enabled again. This feature is used when loading dlls
void disable_registration()
{
	if (!registration_enabled)
		return;
	registration_enabled  = false;
	nr_factories_before_disable = (unsigned int) factories.size();
	nr_objects_before_disable = (unsigned int) objects.size();
}

void emit_events(unsigned int first_factory, unsigned int first_object, abst_registration_listener* listener)
{
	unsigned int i;
	for (i=first_factory; i<factories.size(); ++i) {
		const factory_entry& fe = factories[i];
		listener->register_factory(fe.factory, fe.item_text, fe.shortcut);
	}
	for (i=first_object; i<objects.size(); ++i)
		listener->register_object(objects[i]);
}

/// enable registration and send all registration events that where emitted during disabled registration
void enable_registration()
{
	if (registration_enabled)
		return;
	registration_enabled  = true;
	for (unsigned int i=0; i<listeners.size(); ++i)
		emit_events(nr_factories_before_disable, nr_objects_before_disable, listeners[i]);
}

/// check whether registration is enabled
bool is_registration_enabled()
{
	return registration_enabled;
}

/// add a listener for factory registration events
void add_registration_listener(abst_registration_listener* listener)
{
	listeners.push_back(listener);
	emit_events(0,0,listener);
}

/// register a factor and send event to all current registration listeners
void register_factory(abst_factory* factory, const char* item_text, char shortcut)
{
	factories.push_back(factory_entry(factory,item_text,shortcut));
	if (registration_enabled) {
		for (unsigned int i=0; i<listeners.size(); ++i)
			listeners[i]->register_factory(factory, item_text, shortcut);
	}
}

/// register an object and send event to all current registration listeners
void register_object(base_ptr object)
{
	objects.push_back(object);
	if (registration_enabled) {
		for (unsigned int i=0; i<listeners.size(); ++i)
			listeners[i]->register_object(object);
	}
}

#if defined(_WIN32) && !defined(__CYGWIN__)

#include <windows.h>
#include <winbase.h>

void* load_plugin(const std::string& file_name)
{
	return LoadLibrary(cgv::media::text::str2wstr(file_name).c_str());
}
#else
#if HAVE_DLOPEN
#include <unistd.h>

#ifdef __APPLE__
# include "dlload_osx.cxx"
# define RTLD_NOW 1 // set to anything for now
#else
# include <dlfcn.h>
#endif

void* load_plugin(const std::string& name) {
  // do not allow plugins if this executable is setuid
  if (getuid() != geteuid())  {
    fprintf(stderr, "%s: plugins disabled in setuid programs\n", name);
    return 0;
  }
  void* handle = dlopen(name, RTLD_NOW);
  if (handle) {
    return handle;
  }
  return 0;
}

#else

void* load_plugin(const std::string& name) 
{
  return 0;
}

#endif
#endif

	}
}