#pragma once

namespace cgv {
	namespace data {

/** if you derive your class from this class, a ref_ptr will do reference counting in the inhereted 
    ref_count member. */
class ref_counted
{
private:
	/// keep a mutable reference count to allow ref counted points to const instances
	mutable int ref_count;
public:
	/// constructor initializes the count to 0
	ref_counted() : ref_count(0) {}
	/// read access to current count
	int get_ref_count() const       { return ref_count; }
	/// write access is also const to allow ref counted pointers to const instances
	void set_ref_count(int c) const { ref_count = c; }
};

	}
}