#pragma once

#include <cgv/gui/event.h>

namespace cgv {
	namespace gui {

/// construct event from its kind
event::event(unsigned int _kind, unsigned char _modifiers, unsigned char _toggle_keys, double _time)
	: kind(_kind), modifiers(_modifiers), toggle_keys(_toggle_keys), time(_time) 
{
}

/// virtual destructor for events
event::~event()
{
}
/// write to stream
void event::stream_out(std::ostream& os) const
{
	os << "[" << time << "] ";
	if (get_toggle_keys() != 0) {
		os << "(";
		bool last = false;
		if ((get_toggle_keys()&ETK_CAPS_LOCK)!=0) {
			os << "Caps_Lock";
			last = true;
		}
		if ((get_toggle_keys()&ETK_NUM_LOCK)!=0) {
			if (last)
				os << ",";
			os << "Num_Lock";
			last = true;
		}
		if ((get_toggle_keys()&ETK_SCROLL_LOCK)!=0) {
			if (last)
				os << ",";
			os << "Scroll_Lock";
			last = true;
		}
		os << ") ";
	}
	if ( (get_modifiers()&EM_SHIFT) != 0)
		os << "Shift+";
	if ( (get_modifiers()&EM_ALT) != 0)
		os << "Alt+";
	if ( (get_modifiers()&EM_CTRL) != 0)
		os << "Ctrl+";
	if ( (get_modifiers()&EM_META) != 0)
		os << "Meta+";
	if (kind != EID_KEY && kind != EID_MOUSE)
		os << kind;
}
/// read from stream
void event::stream_in(std::istream& is)
{
	std::cerr << "event::stream_in not implemented yet" << std::endl;
}

/// set the kind of the event
void event::set_kind(unsigned short _kind)
{
	kind = _kind;
}
/// return, what kind of event this is
unsigned int event::get_kind() const
{
	return kind;
}

/// set the state of the toggle keys
void event::set_toggle_keys(unsigned char _toggle_keys)
{
	toggle_keys = _toggle_keys;
}
/// return the state of the toggle keys
unsigned char event::get_toggle_keys() const
{
	return toggle_keys;
}

/// set the modifiers
void event::set_modifiers(unsigned char _modifiers)
{
	modifiers = _modifiers;
}
/// return the active modifiers
unsigned char event::get_modifiers() const
{
	return modifiers;
}
/// set the time of the event
void event::set_time(const double& _time)
{
	time = _time;
}
/// return the time of the event in seconds
double event::get_time() const
{
	return time;
}

	}
}

#include <cgv/config/lib_end.h>