#pragma once

#include <iostream>

#include <cgv/gui/lib_begin.h>

namespace cgv {
	namespace gui {

/// define the ids of the most common events
enum EventId 
{
	EID_NONE,
	EID_KEY,
	EID_MOUSE
};

/// define constants for event modifiers
enum EventModifier 
{
	EM_SHIFT = 1,
	EM_ALT   = 2,
	EM_CTRL  = 4,
	EM_META  = 8
};

/// define constants for event modifiers
enum EventToggleKeys
{
	ETK_CAPS_LOCK = 1,
	ETK_NUM_LOCK   = 2,
	ETK_SCROLL_LOCK  = 4
};

/// most simple event class that can only hold an id. All other event class are derived from this.
class CGV_API event
{
protected:
	/// store which kind of event we have
	unsigned short kind;
	/// store the active modifiers
	unsigned char modifiers;
	/// store the active toggle keys
	unsigned char toggle_keys;
	/// store the time of the event
	double time;
public:
	/// construct event from its kind
	event(unsigned int _kind = EID_NONE, unsigned char _modifiers = 0, unsigned char _toggle_keys = 0, double _time = 0);
	/// virtual destructor for events
	virtual ~event();
	/// write to stream
	virtual void stream_out(std::ostream& os) const;
	/// read from stream
	virtual void stream_in(std::istream& is);
	/// set the kind of the event
	void set_kind(unsigned short _kind);
	/// return, what kind of event this is
	unsigned int get_kind() const;
	/// set the modifiers
	void set_modifiers(unsigned char _modifiers);
	/// return the active modifiers
	unsigned char get_modifiers() const;
	/// set the state of the toggle keys
	void set_toggle_keys(unsigned char _toggle_keys);
	/// return the state of the toggle keys
	unsigned char get_toggle_keys() const;
	/// set the time of the event
	void set_time(const double& _time);
	/// return the time of the event in seconds
	double get_time() const;
};

	}
}

#include <cgv/config/lib_end.h>