#pragma once

#include <cgv/media/font/font_server.h>

#include <cgv/gui/fltk/lib_begin.h>

namespace fltk {
	struct Font;
}

namespace cgv {
	namespace gui {
		namespace fltk {

class fltk_font_face : public media::font::font_face
{
	::fltk::Font* f;
public:
	/// construct from attributes
	fltk_font_face(::fltk::Font* _f, int _ffa);
	/// virtual destructor for ref_ptr support
	~fltk_font_face();
	/// enumerate the supported sizes
	void enumerate_sizes(std::vector<int>& supported_sizes) const;
	/// return the width of a text printed in the given size, which is measured in pixels
	float measure_text_width(const std::string& text, float font_size) const;
	///
	::fltk::Font* get_fltk_font() const;
};

class fltk_font : public media::font::font
{
	::fltk::Font* f;
	int ffa;
public:
	/// 
	fltk_font(::fltk::Font* _f, int _ffa);
	/// virtual destructor for ref_ptr support
	~fltk_font();
	/// return the name of the font
	const char* get_name() const;
	/// check whether the given font includes a face that include the possibly or-ed together selection of font face attributes
	bool supports_font_face(int _ffa) const;
	/// enumerate the supported font sizes
	void enumerate_sizes(std::vector<int>& supported_sizes) const;
	/// return a pointer to a font face
	media::font::font_face_ptr get_font_face(int _ffa) const;
};


/// implements a trigger server with fltk
class CGV_API font_server : public media::font::font_server
{
public:
	/// find an installed font by name
	media::font::font_ptr find_font(const std::string& font_name);
	/// enumerate the names of all installed fonts
	void enumerate_font_names(std::vector<const char*>& font_names);
};

		}
	}
}

#include <cgv/config/lib_end.h>