#include <cgv/gui/fltk/viewer.h>
#include <fltk/TiledGroup.h>
#include <fltk/run.h>
#include <fltk/events.h>
#include <iostream>
#include <math.h>

namespace cgv {
	namespace gui {
		namespace fltk {

using namespace base;

/// construct application
viewer::viewer(int w, int h, const char* name) 
	: Window(::fltk::USEDEFAULT,::fltk::USEDEFAULT,w,h+21,name)
{
	fps = 60;
	time = 0;
	last_clock = 0;
	begin();
		menu = new ::fltk::MenuBar(0, 0, w, 21);
		menu->begin();
			create_item_group = new ::fltk::ItemGroup( "&Create" );
			create_item_group->user_data(this);
		menu->end();
		::fltk::TiledGroup* tile = new ::fltk::TiledGroup(0,21,w,h);
		tile->begin();
			view = gl_view_ptr(new gl_view(0,0,w,h,"view"));
			view->set_ref_count(view->get_ref_count()+1);
			tile->resizable(view);
		tile->end();
		resizable(tile);
	end();
}

void viewer::register_factory(abst_factory* factory, const char* item_text, char shortcut)
{
	if (item_text == 0 && shortcut == 0) {
		register_object(factory->create_object());
		return;
	}
	factories.push_back(factory);
	create_item_group->begin();
	new ::fltk::Item(item_text, ::fltk::COMMAND + shortcut, (::fltk::Callback *)create_cb, (void*)(unsigned int)(factories.size()-1));
	create_item_group->end();
}

void viewer::register_object(base_ptr object)
{
	view->append_child(object);
}

void viewer::create_cb(::fltk::Widget* w, void* val)
{
	::fltk::Item* i = (::fltk::Item*) w;
	::fltk::ItemGroup* g = (::fltk::ItemGroup*) i->parent();	
	viewer* v = (viewer*) g->user_data();
	
	abst_factory* fac = v->factories[(unsigned int)val];
	if (fac->is_singelton) {
		if (fac->singelton)
			v->get_view()->set_focus(fac->singelton);
		else {
			fac->singelton = fac->create_object();
			v->register_object(fac->singelton);
		}
	}
	else
		v->register_object(fac->create_object());
}

/// run the main loop
int viewer::run()
{
	show();
	return ::fltk::run();
}

/*
/// start the game animation
void viewer::start()
{
	time = 0;
	// remember the current time
	last_clock = ::fltk::get_time_secs();
	// trigger a timeout event in order to call the timer_event via the static timeout_callback
	::fltk::add_timeout((float)(1.0/fps), timeout_callback, this);
}

/// in the timer_event event all animations are performed as well as the check for game termination
void viewer::timer_event()
{
	// trigger next timeout event, use the repeat timeout for better timing performance
	::fltk::repeat_timeout((float)(1.0/fps), timeout_callback, this);

	// read the system clock
	double current_clock = ::fltk::get_time_secs();
	// and time difference to last frame
	double dt = current_clock-last_clock;
	// compute current time
	time  += dt;
	// update last clock count
	last_clock = current_clock;
	// perform game animation
	for (unsigned int i=0; i<objects.size(); ++i) {
		timer_event_handler* th = dynamic_cast<timer_event_handler*>(objects[i]);
		if (th)
			if (th->timer_event(time, dt))
				view->redraw();
	}
}
*/
		}
	}
}