#pragma once

#include <cgv/gui/key_event.h>

namespace cgv {
	namespace gui {


/// construct a key event from its textual description 
key_event::key_event(unsigned short _key, KeyAction _action, unsigned char _char, unsigned char _modifiers, unsigned char _toggle_keys, double _time) 
	: event(EID_KEY,_modifiers, _toggle_keys, _time), action(_action), key(_key), character(_char)
{
}

const char* key_strings[] = {
	"F1", 
	"F2", 
	"F3", 
	"F4", 
	"F5", 
	"F6", 
	"F7", 
	"F8", 
	"F9", 
	"F10", 
	"F11", 
	"F12",

	"Space", 
	"Enter", 
	"Tab", 
	"Print",
	"Pause",
	"Break", 
	"Escape", 

	"Left_Shift", 
	"Right_Shift", 
	"Left_Alt", 
	"Right_Alt", 
	"Left_Ctrl", 
	"Right_Ctrl", 
	"Left_Meta", 
	"Right_Meta",

	"Left", 
	"Right", 
	"Up", 
	"Down", 
	"Home", 
	"End", 
	"Page_Up", 
	"Page_Down",

	"Back_Space", 
	"Delete", 
	"Insert",

	"Caps_Lock",
	"Num_Lock",
	"Scroll_Lock",

	"Num_0",
	"Num_1",
	"Num_2",
	"Num_3",
	"Num_4",
	"Num_5",
	"Num_6",
	"Num_7",
	"Num_8",
	"Num_9",
	"Num_Div",
	"Num_Mul",
	"Num_Sub",
	"Num_Add",
	"Num_Dot",
	"Num_Enter",
};

/// write to stream
void key_event::stream_out(std::ostream& os) const
{
	event::stream_out(os);
	if (key > 255)
		os << key_strings[key-256];
	else
		os << (unsigned char) key;
	if (action == KA_RELEASE)
		os << " up";
	else
		if (get_char())
			os << " = '" << get_char() << "'";
}

/// read from stream
void key_event::stream_in(std::istream& is)
{
	std::cerr << "key_event::stream_in not implemented yet" << std::endl;
}

/// 
unsigned short key_event::get_key() const
{
	return key;
}
/// 
void key_event::set_key(unsigned short _key)
{
	key = _key;
}
/// 
unsigned char key_event::get_char() const
{
	return character;
}
/// 
void key_event::set_char(unsigned char _char)
{
	character = _char;
}
/// 
KeyAction key_event::get_action() const
{
	return (KeyAction)action;
}
/// 
void key_event::set_action(KeyAction _action)
{
	action = _action;
}

	}
}
