#pragma once

#include <cgv/gui/event.h>
#include <cgv/gui/lib_begin.h>

namespace cgv {
	namespace gui {

/// define the names of special keys
enum Keys {
	KEY_F1 = 256, 
	KEY_F2, 
	KEY_F3, 
	KEY_F4, 
	KEY_F5, 
	KEY_F6, 
	KEY_F7, 
	KEY_F8, 
	KEY_F9, 
	KEY_F10, 
	KEY_F11, 
	KEY_F12,

	KEY_Space, 
	KEY_Enter, 
	KEY_Tab, 
	KEY_Print, 
	KEY_Pause, 
	KEY_Break, 
	KEY_Escape, 

	KEY_Left_Shift, 
	KEY_Right_Shift, 
	KEY_Left_Alt, 
	KEY_Right_Alt, 
	KEY_Left_Ctrl, 
	KEY_Right_Ctrl, 
	KEY_Left_Meta, 
	KEY_Right_Meta,

	KEY_Left, 
	KEY_Right, 
	KEY_Up, 
	KEY_Down, 
	KEY_Home, 
	KEY_End, 
	KEY_Page_Up, 
	KEY_Page_Down,

	KEY_Back_Space, 
	KEY_Delete, 
	KEY_Insert,

	KEY_Caps_Lock,
	KEY_Num_Lock,
	KEY_Scroll_Lock,

	KEY_Num_0,
	KEY_Num_1,
	KEY_Num_2,
	KEY_Num_3,
	KEY_Num_4,
	KEY_Num_5,
	KEY_Num_6,
	KEY_Num_7,
	KEY_Num_8,
	KEY_Num_9,
	KEY_Num_Div,
	KEY_Num_Mul,
	KEY_Num_Sub,
	KEY_Num_Add,
	KEY_Num_Dot,
	KEY_Num_Enter,
};

/// different actions that a key can perform
enum KeyAction { KA_PRESS, KA_RELEASE };

/// class to represent all possible keyboard events with the EID_KEY
class CGV_API key_event : public event
{
protected:
	/// store the pressed key
	unsigned short key;
	/// store whether 
	unsigned char action;
	/// store the corresponding ascii character
	unsigned char character;
public:
	/// construct a key event from its textual description 
	key_event(unsigned short _key = 0, KeyAction _action = KA_PRESS, unsigned char _char = 0, unsigned char _modifiers = 0, unsigned char _toggle_keys = 0, double _time = 0);
	/// write to stream
	void stream_out(std::ostream& os) const;
	/// read from stream
	void stream_in(std::istream& is);
	/// 
	unsigned short get_key() const;
	/// 
	void set_key(unsigned short _key);
	/// 
	KeyAction get_action() const;
	/// 
	void set_action(KeyAction _action);
	/// 
	unsigned char get_char() const;
	/// 
	void set_char(unsigned char _char);
};

	}
}

#include <cgv/config/lib_end.h>