#pragma once

#include <cgv/gui/mouse_event.h>

namespace cgv {
	namespace gui {


/// construct a key event from its textual description 
mouse_event::mouse_event(int _x, int _y, MouseAction _action, unsigned char _button_state, unsigned char _button, short _dx, short _dy, unsigned char _modifiers, unsigned char _toggle_keys, double _time)
	: event(EID_MOUSE,_modifiers,_toggle_keys,_time), x(_x), y(_y), action(_action), button_state(_button_state), button(_button), dx(_dx), dy(_dy)
{
}
/// write to stream
void mouse_event::stream_out(std::ostream& os) const
{
	const char* action_strs[] = {
		"press", "release", "wheel", "move", "drag", "enter", "leave"
	};
	const char* button_strs[] = {
		"", "Left_Button", "Middle_Button", "", "Right_Button"
	};
	event::stream_out(os);
	if ((get_button_state()&MB_LEFT_BUTTON)!=0)
		os << "Left_Button+";
	if ((get_button_state()&MB_MIDDLE_BUTTON)!=0)
		os << "Middle_Button+";
	if ((get_button_state()&MB_RIGHT_BUTTON)!=0)
		os << "Right_Button+";
	os << "mouse " << action_strs[get_action()];
	if (get_action() == MA_PRESS || get_action() == MA_RELEASE)
		os << " " << button_strs[get_button()];
	if (get_action() == MA_WHEEL)
		os << "(" << get_dy() << ")";
	os << " at " << get_x() << "," << get_y();
	if (get_action() == MA_DRAG || get_action() == MA_MOVE) 
		os << "(" << get_dx() << "," << get_dy() << ")";
}
/// read from stream
void mouse_event::stream_in(std::istream& is)
{
}
/// 
short mouse_event::get_x() const
{
	return x;
}
/// 
short mouse_event::get_y() const
{
	return y;
}
/// not used yet
short mouse_event::get_dx() const
{
	return dx;
}
/// used for mouse wheel delta
short mouse_event::get_dy() const
{
	return dy;
}
/// 
MouseAction mouse_event::get_action() const
{
	return (MouseAction)action;
}
/// 
unsigned char mouse_event::get_button_state() const
{
	return button_state;
}
/// 
unsigned char mouse_event::get_button() const
{
	return button;
}

	}
}
