#pragma once

#include <cgv/gui/event.h>
#include <cgv/gui/lib_begin.h>

namespace cgv {
	namespace gui {


/// different actions that a mouse can perform
enum MouseAction { MA_PRESS, MA_RELEASE, MA_WHEEL, MA_MOVE, MA_DRAG, MA_ENTER, MA_LEAVE };
/// different mouse buttons that can be ored together to specify the button state
enum MouseButton { 
	MB_NO_BUTTON = 0, 
	MB_LEFT_BUTTON = 1, 
	MB_MIDDLE_BUTTON = 2, 
	MB_RIGHT_BUTTON = 4 };

/// class to represent all possible mouse events with the EID_MOUSE
class CGV_API mouse_event : public event
{
protected:
	/// position of mouse pointer
	short x,y;
	/// change in position
	short dx,dy;
	/// store whether 
	unsigned char action;
	/// store the button state
	unsigned char button_state;
	/// store the pressed button
	unsigned char button;
public:
	/// construct a mouse
	mouse_event(int x, int y, MouseAction _action, unsigned char _button_state = 0, unsigned char _button = 0, short _dx = 0, short _dy = 0, unsigned char _modifiers = 0, unsigned char _toggle_keys = 0, double _time = 0);
	/// write to stream
	void stream_out(std::ostream& os) const;
	/// read from stream
	void stream_in(std::istream& is);
	/// current mouse x position 
	short get_x() const;
	/// current mouse y position 
	short get_y() const;
	/// for move and drag events the difference in x to the previous position
	short get_dx() const;
	/// for move and drag events the difference in y to the previous position, for wheel events the amount the wheel has changed
	short get_dy() const;
	/// 
	MouseAction get_action() const;
	/// 
	unsigned char get_button_state() const;
	/// 
	unsigned char get_button() const;
};

	}
}

#include <cgv/config/lib_end.h>