#include <cgv/gui/trigger.h>
#include <iostream>

namespace cgv {
	namespace gui {

/// default construction
trigger::trigger()
{
	delay = 0;
	recur = false;
}
/// schedule a trigger event in delay seconds
void trigger::schedule_on_shot(double _delay)
{
	delay = _delay;
	recur = false;
	trigger_server* ts = get_trigger_server();
	if (ts)
		ts->schedule_on_shot(this, delay);
	else
		std::cerr << "no trigger server registered" << std::endl;
}

/// start a recuring trigger with the given delay in seconds
void trigger::schedule_recuring(double _delay)
{
	delay = _delay;
	recur = true;
	trigger_server* ts = get_trigger_server();
	if (ts)
		ts->schedule_recuring(this, delay);
	else
		std::cerr << "no trigger server registered" << std::endl;
}
/// return the delay used for scheduling the last time
double trigger::get_delay() const
{
	return delay;
}

/// return whether this is a recuring trigger
bool trigger::is_recuring() const
{
	return recur; 
}

/// stop a recuring trigger
void trigger::stop()
{
	trigger_server* ts = get_trigger_server();
	if (ts)
		ts->stop(this);
	else
		std::cerr << "no trigger server registered" << std::endl;
}

/// return the global trigger used for animation, which runs by default with 60 Hz
trigger& get_animation_trigger()
{
	static trigger t;
	static bool initialized = false;
	if (!initialized) {
		initialized = true;
		t.schedule_recuring(1.0/60);
	}
	return t;
}

trigger_server* ts = 0;

trigger_server* get_trigger_server()
{
	return ts;
}

void set_trigger_server(trigger_server* _ts)
{
	ts = _ts;
}

	}
}
