#pragma once

#include <cgv/base/signal.h>

#include <cgv/gui/lib_begin.h>

namespace cgv {
	namespace gui {

/// a trigger provides callbacks after a programmable time delay
class CGV_API trigger
{
protected:
	double delay;
	bool recur;
public:
	/// default construction
	trigger();
	/// schedule a trigger event in delay seconds
	void schedule_on_shot(double delay);
	/// start a recuring trigger with the given delay in seconds
	void schedule_recuring(double delay);
	/// return the delay used for scheduling the last time
	double get_delay() const;
	/// return whether this is a recuring trigger
	bool is_recuring() const;
	/// stop a recuring trigger
	void stop();
	/// the shoot signal is called when the trigger is pulled and takes the current time and delay as argument
	base::signal<double,double> shoot;
};

/// return the global trigger used for animation, which runs by default with 60 Hz
extern CGV_API trigger& get_animation_trigger();

class CGV_API trigger_server
{
public:
	virtual void schedule_on_shot(trigger*, double delay) = 0;
	virtual void schedule_recuring(trigger*, double delay) = 0;
	virtual void stop(trigger*) = 0;
};
extern CGV_API trigger_server* get_trigger_server();
extern CGV_API void set_trigger_server(trigger_server*);

	}
}

#include <cgv/config/lib_end.h>