#pragma once
#include <cgv/math/mat.h>
#include <cgv/math/perm_mat.h>
#include <cgv/math/diag_mat.h>
#include <cgv/math/low_tri_mat.h>
#include <cgv/math/up_tri_mat.h>
#include <cgv/math/lu.h>

namespace cgv {
namespace math {

//compute determinant of a diagonal matrix
template <typename T>
T det(const diag_mat<T>& m) 
{
	T v = (T)1.0;
	for(unsigned i =0; i< m.nrows(); i++)
		v *= m(i);
	return v;
}

//compute determinant of an lower triangular matrix
template <typename T>
T det(const low_tri_mat<T>& m) 
{
	T v = (T)1.0;
	for(unsigned i =0; i< m.nrows(); i++)
		v *= m(i,i);
	return v;
}

//compute determinant of an upper triangular matrix
template <typename T>
T det(const up_tri_mat<T>& m) 
{
	T v = (T)1.0;
	for(unsigned i =0; i< m.nrows(); i++)
		v *= m(i,i);
	return v;
}

//compute determinant of a permutation matrix
inline int det(const perm_mat& m) 
{
	//count number of transpositions needed to get the identity permutation
	perm_mat temp = m;
	unsigned num_transpositions = 0;
	for(unsigned i = 0; i < m.size();i++)
	{
		while(temp(i) != i)
		{
			std::swap(temp(i),temp(temp(i)));
			num_transpositions++;
		}
	}
	//if number of transpositions is even return 1 else -1
	if(num_transpositions % 2 == 0)
		return 1;
	else 
		return -1;
	
}

//compute determinant of a square matrix m
template <typename T>
T det(const mat<T>& m) 
{
	assert(m.is_square());
	perm_mat p;
	low_tri_mat<T> l;
	up_tri_mat<T> u;
	lu(m,p,l,u);
	return det(p)*det(l)*det(u);
}

}

}


