#pragma once

#include <cgv/math/mat.h>

#include <cgv/math/diag_mat.h>

#include <limits>
#include <algorithm>

namespace cgv {
namespace math {
	 
template <typename T>
void rot(mat<T> &a, const T s, const T tau, const int i,
		const int j, const int k, const int l)
	{
		T g=a(i,j);
		T h=a(k,l);
		a(i,j)=g-s*(h+g*tau);
		a(k,l)=h+s*(g-h*tau);
	}


template <typename T>
void eigsrt(diag_mat<T> &d, mat<T> &v)
{
	unsigned k;
	unsigned n=d.size();
	for (unsigned i=0;i<n-1;i++) 
	{
		T p=d(k=i);
		for (unsigned j=i;j<n;j++)
			if (d(j) >= p) p=d(k=j);
		if (k != i) 
		{
			d(k)=d(i);
			d(i)=p;
			
			for (unsigned j=0;j<n;j++) 
			{
					p=v(j,i);
					v(j,i)=v(j,k);
					v(j,k)=p;
			}
		}
	}
}

///eigen decomposition of a symmetric matrix using the jacobi method
///v contains the eigenvectors
///d contains the eigenvalues
///a=v*d*transpose(v)
template <typename T>
bool eig_sym(const mat<T> &a, mat<T> &v, diag_mat<T> &d,bool ordering=true, unsigned maxiter=50)
{
	mat<T>aa = a;
	unsigned n =aa.nrows();
	v.identity(n);
	d.resize(n);
	unsigned nrot=0;
	const double eps = std::numeric_limits<T>::epsilon();
	T tresh,theta,tau,t,sm,s,h,g,c;
	vec<T> b(n),z;
	z.zeros(n);
	unsigned ip,iq;
	
	for(unsigned i = 0; i < n; i++)
		d(i)=b(i)=aa(i,i);

	for (unsigned i=1;i<=maxiter;i++) 
	{
			sm=(T)0.0;
			
			for (ip=0;ip<n-1;ip++) 
			{
				for (iq=ip+1;iq<n;iq++)
					sm += std::abs(aa(ip,iq));
			}
			if (sm == (T)0.0) 
			{
				if(ordering)
					eigsrt(d,v);
				return true;
			}
			if (i < 4)
				tresh=(T)(0.2*sm/(n*n));
			else
				tresh=(T)0.0;
			for (ip=0;ip<n-1;ip++) 
			{
				for (iq=ip+1;iq<n;iq++) 
				{
					g=((T)100.0)*std::abs(aa(ip,iq));
					if (i > 4 && g <= eps*std::abs(d(ip)) && g <= eps*std::abs(d(iq)))
							aa(ip,iq)=(T)0.0;
					else if (std::abs(aa(ip,iq)) > tresh) 
					{
						h=d(iq)-d(ip);
						if (g <= eps*std::abs(h))
							t=(aa(ip,iq))/h;
						else {
							theta=(T)(0.5*h/(aa(ip,iq)));
							t=(T)(1.0/(std::abs(theta)+sqrt(1.0+theta*theta)));
							if (theta < 0.0) t = -t;
						}
						c=(T)(1.0/sqrt(1+t*t));
						s=t*c;
						tau=(T)(s/(1.0+c));
						h=t*aa(ip,iq);
						z(ip) -= h;
						z(iq) += h;
						d(ip) -= h;
						d(iq) += h;
						aa(ip,iq)=(T)0.0;
						for (unsigned j=0;j<ip;j++)
							rot(aa,s,tau,j,ip,j,iq);	
						for (unsigned j=ip+1;j<iq;j++)
							rot(aa,s,tau,ip,j,j,iq);
						for (unsigned j=iq+1;j<n;j++)
							rot(aa,s,tau,ip,j,iq,j);
						for (unsigned j=0;j<n;j++)
							rot(v,s,tau,j,ip,j,iq);
						++nrot;
					}
				}
			}
			for (ip=0;ip<n;ip++) {
				b(ip) += z(ip);
				d(ip)=b(ip);
				z(ip)=(T)0.0;
			}
		}
		return false;
		//Too many iterations in routine jacobi
	
}

}}