#pragma once
#include <cgv/math/lin_solve.h>

namespace cgv {
	namespace math {


//returns the inverse of a diagonal matrix
template <typename T>
diag_mat<T> inv(const diag_mat<T>& m) 
{
	diag_mat<T> im(m.size());
	for(unsigned i = 0; i < m.nrows();i++)
		im(i) = (T)(1.0/m(i));
	return im;
}

//returns the inverse of a lower triangular matrix
template <typename T>
low_tri_mat<T> inv(const low_tri_mat<T>& m) 
{
	unsigned N = m.nrows();
	low_tri_mat<T> im(N,(T)0.0);
	T sum;
	for(unsigned k = 0; k < N;k++)
	{	
		for(unsigned i = k; i < N;i++)
		{
			sum =0;
			for(unsigned j = k;j < i; j++) 
				sum += m(i,j)*im(j,k);
			assert(m(i,i) != 0);// not invertible
			if(i == k) 
				im(i,k) = (1 - sum)/m(i,i);
			else
				im(i,k) = ( -sum)/m(i,i);
		}
	}
	
	
	return im;
}

//returns the inverse of an upper triangular matrix
template <typename T>
up_tri_mat<T> inv(const up_tri_mat<T>& m) 
{
	up_tri_mat<T> im(m.nrows(),(T)0.0);
	unsigned N = m.nrows();
	vec<double> x;
	vec<double> b(N);
	x.resize(N);
	T sum;
	for(unsigned k = 0; k < N;k++)
	{
		for(int i = k; i >= 0;i--)
		{
			sum =0;
			for(unsigned j = i+1;j <= k; j++) 
				sum += m(i,j)*im(j,k);
			assert(m(i,i) != 0);
				
			
			if(k == i)
				im(i,k) = ((T)1.0 - sum)/m(i,i);
			else
				im(i,k) = ( - sum)/m(i,i);
		}
	}
	return im;
}

template <typename T>
mat<T> inv(const mat<T>& m) 
{
	assert(m.nrows() == m.ncols());
	mat<T> im(m.nrows(),m.nrows());
	solve(m,identity<T>(m.ncols()),im);
	return im;
}


inline const perm_mat inv(const perm_mat &p)
{
	perm_mat r=p;
	r.transpose();
	return r;
}





	}
}
