#pragma once

#include <stack>

#include <cgv/media/text/callback_stream.h>
#include <cgv/render/context.h>
#include <cgv/render/gl/lib_begin.h>

namespace cgv {
	namespace render {
		namespace gl {

/** implementation of the context API for the OpenGL API excluding methods for font selection, redraw and
    initiation of render passes. */
class CGV_API context : public render::context, public cgv::base::tacker
{
public:
	/// construct context and attach signals
	context();
	/// return the used rendering API
	RenderAPI get_render_api() const;

	/**@name text output*/
	//@{
	/// use this to push transformation matrices on the stack such that x and y coordinates correspond to window coordinates
	void push_pixel_coords();
	/// set a new cursor position, which is only valid between calls of push_pixel_coords and pop_pixel_coords
	void set_cursor(int x, int y);
	/// store the current cursor location in the passed references to x and y coordinate
	void get_cursor(int& x, int& y) const;
	/// pop previously changed transformation matrices 
	void pop_pixel_coords();
	//@}

	/**@name tesselation*/
	//@{
	/// tesselate with independent faces
	virtual void draw_faces(
		const double* vertices, const double* normals, const double* tex_coords, 
		const int* vertex_indices, const int* normal_indices, const int* tex_coord_indices, 
		int nr_faces, int face_degree, bool flip_normals) const;
	/// tesselate with one face strip
	virtual void draw_strip_or_fan(
		const double* vertices, const double* normals, const double* tex_coords, 
		const int* vertex_indices, const int* normal_indices, const int* tex_coord_indices, 
		int nr_faces, int face_degree, bool is_fan, bool flip_normals) const;
	//@}

	/**@name transformations*/
	//@{
	/// return homogeneous 4x4 viewing matrix, which transforms from world to eye space
	const mat_type get_V() const;
	/// return homogeneous 4x4 projection matrix, which transforms from eye to clip space
	const mat_type get_P() const;
	/// return homogeneous 4x4 projection matrix, which transforms from clip to device space
	const mat_type get_D() const;
	/// read the device z-coordinate from the z-buffer for the given device x- and y-coordinates
	double get_z_D(int x_D, int y_D) const;
	//@}
};
		}
	}
}