/*-------------------------------------------------------------

Simple implementation of Marching Squares algorithm

----------------------------------------------------------------*/


#include "MarchSquares.h"


namespace MarchSquares
{

	double getInterpolationValue(double val1, double val2, double iso) // helper function: returns the normalized value of iso level inbetween interval of two scalars
	{
		double result = (val1 - iso)/(val1+val2);
		
		if (result < 0.0) 
			result = 0.0;
		
		if (result > 1.0)
			result = 1.0;
		return result;
	}

	double interpolateLinear(double val1, double val2, double t) //helper function: returns affin combination of two scalars with parameter t
	{
		return (1 - t) * val1 + t * val2;
	}


	void renderMS(double iso, std::vector<float> data, int DimU, int DimV) // the actual march squares implementation
	{		
		GLuint id = glGenLists(1);
		glNewList(id, GL_COMPILE);
		glBegin(GL_LINES);

		for(int u = 0; u < DimU-1; u++)
			for(int v = 0; v < DimV-1; v++)
			{
				double a,b,c,d;

				a = data[v*DimU + u];
				b = data[(v+1)*DimU + u];
				c = data[(v+1)*DimU + u + 1];
				d = data[v*DimU + u + 1];

				int code=0;
			
				(a > iso)?(code += 2000):(code += 1000);  //We provide a code describing the scalar field values at the cell corners in respect to the iso value
				(b > iso)?(code += 200):(code += 100);    //where 1 means less or equal and 2 means greater
				(c > iso)?(code += 20):(code += 10);	  //(should properly done with hexadecimal values)
				(d > iso)?(code += 2):(code += 1);
						
				double t;
				double x,y;
				double temp;

				int ba = 0, bc = 0, cd = 0, ad = 0, abcd = 0;  // these variables describe if there is an intersection of iso lines with current cell edges, "abcd" is a special
															   // case where normal rendering order has to be changed

				switch(code) //with the help of the code we can easily find the intersected cell borders
				{

					case 2221:
					case 1112:
						ad = 1;
						cd = 1;
						break;

					case 2212:
					case 1121:						
						bc = 1;
						cd = 1;
						break;

					case 2211:
					case 1122:						
						ad = 1;
						bc = 1;
						break;

					case 2122:
					case 1211:						
						ba = 1;
						bc = 1;
						break;

					case 2121:
						temp = ((a+d)/2.0 + (b+c)/2.0)/2.0;
						if (temp > iso)
						{
							ba = 1;
							bc = 1;
							cd = 1;
							ad = 1;
						}
						else abcd = 1;
						break;

					case 1212:
						
						temp = ((a+d)/2.0 + (b+c)/2.0)/2.0;
						if (temp <= iso)
						{
							ba = 1;
							bc = 1;
							cd = 1;
							ad = 1;
						}
						else abcd = 1;
						break;

					case 2112:
					case 1221:						
						ba = 1;
						cd = 1;
						break;
					
					case 2111:
					case 1222:						
						ba = 1;
						ad = 1;
						break;
				}
				
				
				// in the following we use the special order of helper variables to simplify rendering in such a way that we needn't do vertex2D calls for every line segment itself
				if (ad == 1)
				{
					t = getInterpolationValue(a,d,iso);
					x = interpolateLinear(u,u+1,t);
					y = v;
					glVertex2d(x,y);
				}

				if (cd == 1)
				{
					t = getInterpolationValue(c, d,iso);
					x = u+1;
					y = interpolateLinear(v+1,v,t);
					glVertex2d(x,y);
				}

				if (ba == 1)
				{
					t = getInterpolationValue(b, a,iso);
					x = u;
					y = interpolateLinear(v+1,v,t);
					glVertex2d(x,y);
				}				

				if (bc == 1)
				{
					t = getInterpolationValue(b,c,iso);
					x = interpolateLinear(u,u+1,t);
					y = v+1;
					glVertex2d(x,y);
				}

				if (abcd == 1) //only case where ordering runs fail
				{
					t = getInterpolationValue(b, a,iso);
					x = u;
					y = interpolateLinear(v+1,v,t);
					glVertex2d(x,y);

					t = getInterpolationValue(a,d,iso);
					x = interpolateLinear(u,u+1,t);
					y = v;
					glVertex2d(x,y);

					t = getInterpolationValue(b,c,iso);
					x = interpolateLinear(u,u+1,t);
					y = v+1;
					glVertex2d(x,y);

					t = getInterpolationValue(c, d,iso);
					x = u+1;
					y = interpolateLinear(v+1,v,t);
					glVertex2d(x,y);
				}				
			}
			glEnd();
			glEndList();
			glCallList(id);
			glDeleteLists(id,1);
	}
}