#pragma once
#include "pca/PCAfluid.h"
#include "util/Image.h"
#include <vector>
#include <windows.h>
#include <glut.h>
#include "MarchSquares.h"
#include "MarchCubes.h"
#include "util/Camera.h"
#include "util/DebugNavigator.h"


class RenderManager
{
private:
	bool DirtyFlag; //used when we have to do a redraw after parameter changes
	pca::PCAfluid* PCAComponent; // the render manager administrates it's own pointer to PCA object 
	
	//textures, IDs and other stuff needed for rendering
	unsigned int originalDataTexID, reconstructedDataTexID;
	util::Image originalDataTexture, reconstructedDataTexture;
	int WindowWidth, WindowHeight;
	unsigned int MCDisplayListID1;
	unsigned int MCDisplayListID2;
	util::Camera camera;  

	// some color specs
	GLfloat PropertiesAmbient[4]; 
	GLfloat PropertiesDiffuse[4]; 
	GLfloat PropertiesSpecular[4]; 
	GLfloat Color[4]; 
		

public:
	util::DebugNavigator viewManipulator; // camera control

	RenderManager(int wwidth, int wheight);
	~RenderManager();


	void setMCEnvironment(); // initial illumination, material and color settings	
	void regenerateTextureIDs(); //delete old textures, free up the memory and generate new ones
	//generate textures
	void buildTextureFromDataVector(int width, int height, const std::vector<float> &dataVector1, const std::vector<float> &dataVector2, bool clamp);
	void buildTextureFromDataVector(int width, int height, util::Image &texture, unsigned int &texID, const std::vector<float> &dataVector, bool clamp);	
	void rebuildTextures(int DataSetNr, bool showRecalculatedData, int princCompNr, bool useClamping);
	void setDirtyFlag(); //force a redraw
	
	//different render methods
	void renderTextures(); 	

	void renderMarchSquares(int DataSetNr, double IsoLevel);
	void renderMarchSquares(std::vector<float> dataleft, std::vector<float> dataright, double IsoLevel);
	void renderMarchCubes(std::vector<float> dataleft, std::vector<float> dataright, double IsoLevel);
	
	void renderMarchCubes(int DataSetNr, double IsoLevel);
	void renderMarchCubes(float DataSetNr,double IsoLevel, int showInfos, int PrincCompNr);

	void renderFeatureVector(int nr, int CurrentSet);
	void renderInterpolatedData(float DataSetNr, int showTextures, int showMarchingSquaresOverlay, int showInfo, int showReconstructedData, double IsoLevel, int PrincCompNr, bool clamp); //render interpolated data immediately
			
	void renderTextLine(float XPos, float YPos, void *font, std::string text, float r, float g, float b);	
	void renderTextLine(float XPos, float YPos, std::string text);

	void showInfos(bool showRecalculatedData, int DataSetNr, double IsoLevel, int princCompNr);


	// helper functions
	void setWindowSize(int width, int height);
	void setPCAComponent(pca::PCAfluid *component);
	pca::PCAfluid* getPCAComponent();
	
};