/*---------------------------------------------------------------------

Simple Oscillator Class that we need for our simulation model

----------------------------------------------------------------------*/
#pragma once

#define PI   3.141592653589793f

class Oscillator
{
public: 

	float _Time;
	float _StepSize;
	int   _NrOfHarmonics;
	float _HarmonicDamping;
	float _AmplitudeDamping;
	float _Amplitude;
	float _Phase;
	float _FundamentalFrequency;
	int _NrOfPC;
	float _DC;
	int _ID;
	GLUI_Rollout * _GUIPtr;
	

	Oscillator(int nh, float hd, float ad,	float a, float p, float ff, float dc, int nop)
	{
		_NrOfHarmonics = nh;
		_HarmonicDamping = hd;
		_AmplitudeDamping = ad;
		_Amplitude=a;
		_Phase = p;
		_FundamentalFrequency = ff;
		_Time = 0.0f;
		_StepSize = 0.01f;
		_DC = dc;
		_NrOfPC = nop;
	}

	Oscillator()
	{
		_NrOfHarmonics = 1;
		_HarmonicDamping = 0.0f;
		_AmplitudeDamping = 0.0f;
		_Amplitude=0.0f;
		_Phase = 0.0f;
		_FundamentalFrequency = 1.0f;
		_Time = 0.0f;
		_StepSize = 0.01f;
		_DC = 0.0f;
		_NrOfPC = 0;
	}

	
	void step()
	{
		_Time += _StepSize;
	}

	float getValue() // returns the result of the oscillator model evaluated at current point in time 
	{
		
		float temp = 0.0f;		

		for(int i=0; i<_NrOfHarmonics; i++)
		{
			temp +=  exp(-_HarmonicDamping*i) * sinf(2*PI*(i+1)*_FundamentalFrequency*_Time + _Phase);			
		}
		return exp(-_AmplitudeDamping*_Time) * temp  * _Amplitude + _DC;
		
	}
};