/*---------------------------------------------------------------------



----------------------------------------------------------------------*/
#include <cmath>
#include <string>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <sstream>
#include <ostream>
#include <cstdarg>
#include "grid/DiscreteCartesian_2d.h"
#include "grid/BaseCell_2d.h"
#include "grid/BoundedGrid_2d.h"
#include "grid/fast_sweep_2d.h"
#include "util/Camera.h"
#include "util/DebugNavigator.h"
#include "util/Image.h"
#include "util/FileSequence.h"
#include "CustomCell.h"
#include <vector>
#include <deque>
#include <sstream>


#include "pca/PCAfluid.h"
#include <windows.h>
#include <glut.h>
#include <glui.h>
#include "rendering/RenderManager.h"
#include "util/Oscillator.h"


#define LOAD					 1
#define SAVE					 2
#define IMPORT					 3
#define SHOWFEATCHECKBOX         4
#define APPLYREC			     5
#define CURDATASETSPINNER		 6
#define CURFEATURE			     7
#define RECONSTRUCTIONRADIOGROUP 8
#define PCSPINNER				 9
#define ISOSPINNER				10
#define SHOWTEXTURESCHECKBOX	11
#define SHOWEIGENVALUESCHECKBOX 12
#define APPLYBOUND			    13
#define LOWERPCSPINNER			14
#define UPPERPCSPINNER			15
#define EIGENVALUESCLOSE		16
#define FILEBROWSERCLICK		17
#define CANCELBUTTON			18
#define SHOWERRORWINDOWCHECKB	19
#define CLOSEERRORWINDOWBUTTON	20
#define TIMESTEPSPINNER			21
#define TIMESTEPCHECKBOX		22
#define RESETSIMUBUTTON			23
#define OSCILLATORBUTTON		24
#define STARTSIMUBUTTON			25
#define SIMUTIMESTEPSPINNER		26
#define ADDOSCILLATORBUTTON		27
#define REMOVEOSCBUTTON			28



#define ERRORFILE		"MSE.log"
#define VARIANCEFILE    "Variance.log"
#define FEATUREVECTORFILE  "Featurevectors.log"
#define MINMAXMEANFEATUREAMPLITUDE "MinMaxMeanFeatureAmp.log"




// helper class to load the data
class CustomCell;
typedef grid::BoundedGrid_2d<grid::DiscreteCartesian_2d, CustomCell>         Grid;



// global variables for file functions
util::FileSequence fileSequence;  // sequence/animation of grid files
util::FileSequence::iterator currentFile;



// global variables for analysis
int PrincCompNr = 1;
int PrincCompForRec = 2;
int FeatureNr = 1;

// global variables for rendering and window positioning
int lastMousePosX = 0;
int lastMousePosY = 0;
int lastFVWPosX = 0;
int lastFVWPosY = 0;
int lastFVWSizeX = 600;
int lastFVWSizeY = 300;
int lastEVPosX = 0;
int lastEVPosY = 0;
int MainWindow = 0;
int FeatureVectorWindow = 0;
int WindowHeight = 600;
int WindowWidth = 1200;

RenderManager* myRenderManager = new RenderManager(WindowWidth, WindowHeight);

enum STATE {ROTATING, ZOOMING, PANNING, INACTIVE, MODE_3D, MODE_2D};
STATE cameraState = INACTIVE;

float IsoLevel = 0.0;
void draw();
void redraw();
const double getIsoLevel();
void setIsoLevel(double value);


// global variables for animation
int DataSetNr = 0; // holds the current data set
float InterpolDataSetNr = 0.0f; // holds the current time for interpolated data set
int animateFlag = 0;
void animate(int flag);
void nextFrame();
void prevFrame();


//Variables used for menu interaction

int RadioButtonSelection = 0;
int ImportRadioSelection = 0;
int lowerPC = 0, upperPC = 1;
int mode = 0; // 0->2D, 1->3D
int usePrefiltering = 1; // prefiltering while importing files
float PrefilterThreshold = 0.001f; // threshold value for prefiltering
int useTransposing = 0; // transpose data matrix before doing the eigenvalue decomposition (good for large data)
int useNormalizing = 0; //normalize data to interval [-1,1]
int useInterpolationMode = 0; 
int showReconstructedData = 1;
int showMarchingSquaresOverlay = 0;
int showTextures = 1;
int showFeatures = 0;
int showInfos = 1;
int showEigenvalues = 0;
int showErrorWindow = 0;
int useRenderClamping = 1;
float timeStep = 1.000;


GLUI *Controls; // main menu
GLUI *EigenvalueWindow;
GLUI_Spinner *NrOfPCSpinner;
GLUI_Spinner *LowerBoundPCSpinner;
GLUI_Spinner *UpperBoundPCSpinner;
GLUI_Spinner *PCSpinner;
GLUI_StaticText *TotalNrOfDataSets;
GLUI_Spinner *CurDataSetSpinner;
GLUI_Spinner *FeatureSpinner;
GLUI_RadioGroup *ReconstructionGroup;
GLUI_Button *ApplyButton;
GLUI_Button *ApplyBoundButton;
GLUI_FileBrowser *ImportFileBrowser;
GLUI_RadioGroup *ImportRadioGroup;
GLUI *ImportWindow;
GLUI *ErrorWindow;
GLUI_TextBox *ErrorTextBox;
GLUI_Spinner *PrefilterSpinner;




// EXPERIMENTAL: OSCILLATION MODEL 

void simulateStep();
void SimulationControl(int ID);
void resetOscillators();


float SimuTime = 0.0f;
float SimuTimeStepSize = 0.03f;
GLUI_EditText *SimuTimeText;
GLUI_Panel *OscillatorPanel;

int SimulationFlag = 0;
int SimuID = 0;
int getSimuID()
{
	return SimuID++;
}


void createOscillatorGUI(Oscillator *osc, int ID);
std::vector<Oscillator*> OscillatorList;






/* ------------- FILE FUNCTIONS ------------------------------------ */
void getDataExtends(const char *nameOfFile, int &dimX, int &dimY)  // get the x- and y-dimensions of a 2D data set
{
	std::ifstream file;
	file.open( nameOfFile, std::ios::in | std::ios::binary );
	float cellSize; // not used

	// read cellSize
	file.read( (char*)&cellSize,sizeof(float));
	// read number of voxels in x
	file.read( (char*)&dimX,sizeof(unsigned int));
	// read number of voxels in y
	file.read( (char*)&dimY,sizeof(unsigned int));

	
}

void getDataExtends3D(const char *nameOfFile, unsigned int &dimX, unsigned int &dimY, unsigned int &dimZ)  // get the x-, y-, and z-dimensions of a 3D data set
{
	
	std::ifstream file;
	file.open( nameOfFile, std::ios::in | std::ios::binary );
	float cellSize; // not used
	
	// read cellSize
	file.read( (char*)&cellSize,sizeof(float));
	// read number of voxels in x
	file.read( (char*)&dimX,sizeof(unsigned int));
	// read number of voxels in y
	file.read( (char*)&dimY,sizeof(unsigned int));
	// read number of voxels in z
	file.read( (char*)&dimZ,sizeof(unsigned int));

	file.close();

}

// write data from file into variable data and apply filter if neccessary
void importGrid2D(std::string nameOfFile, std::vector<float> *data, bool useFilter, float Threshold)
{
	std::ifstream file;
	file.open( (nameOfFile).c_str(), std::ios::in | std::ios::binary );

	Grid GridData;
	file>>GridData;
	file.close();
	
	data->reserve(GridData.dimensionX()*GridData.dimensionY());

	// iterate through data set 
	for( Grid::iterator it = GridData.begin(); it != GridData.end(); ++it )
	{
		//filter values to -1,0,+1
		if (useFilter)
		{
			if (it->signedDistance() > Threshold)
				data->push_back(1.0f);
			else if (it->signedDistance() < -Threshold)
				(data->push_back(-1.0f));
			else 
				(data->push_back(0.0f));
		}


		// don't filter
		else
		{
			data->push_back(it->signedDistance());
		}
	
	}
}

// import a 3d dataset, write from file into variable voxels
void importGrid3d( const char *nameOfFile, std::vector<float> &voxels,  unsigned int &dimX, unsigned int &dimY, unsigned int &dimZ)
{
	
	std::ifstream file;
	file.open( nameOfFile, std::ios::in | std::ios::binary );

	float cellSize; // not used


	// read cellSize
	file.read( (char*)&cellSize,sizeof(float));
	// read number of voxels in x
	file.read( (char*)&dimX,sizeof(unsigned int));
	// read number of voxels in y
	file.read( (char*)&dimY,sizeof(unsigned int));
	// read number of voxels in z
	file.read( (char*)&dimZ,sizeof(unsigned int));

	// prepare voxels
	unsigned int numVoxels = dimX*dimY*dimZ;
	voxels.resize( numVoxels );

	// read data (all voxels)
	for( unsigned int i=0; i<numVoxels; ++i )
	{
		// read current voxel
		file.read( (char*)&voxels[i],sizeof(float));
	}

	// done
	file.close();
}


// create string and return the calculated error
std::string getErrorText()
{
	pca::PCAfluid* pca = myRenderManager->getPCAComponent();
	//create content string
	std::stringstream ss;
	
	ss<<"Average MSE: \t\t\t"<<pca->getMeanReconstructionError()<<"\n";
	ss<<"Minimum MSE: \t\t\t"<<pca->getMinReconstructionError()<<"\n";
	ss<<"Maximum MSE: \t\t\t"<<pca->getMaxReconstructionError()<<"\n";
	ss<<"MSE of current data set: \t\t"<<pca->getReconstructionError(DataSetNr)<<"\n";
	
	/*
	ss<<pca->getMeanReconstructionError()<<"\t\t";
	ss<<pca->getMinReconstructionError()<<"\t\t";
	ss<<pca->getMaxReconstructionError()<<"\t\t";   */       //<- use these lines when writing errors to file 
	

	return ss.str();

}


// create content string for variance window
std::string getVarianceText()
{
	std::stringstream ss;
	ss<<"Weight of PC \t\t Accumulative Weight\n";
	float sum = 0.0f;
	for(unsigned int i=0; i < myRenderManager->getPCAComponent()->getEigenValues().size(); i++)
	{
		float EigenValue = (float)((int)(myRenderManager->getPCAComponent()->getEigenValues()[i]*10000))/100;
		sum += EigenValue;
		ss <<(i+1)<<". PC: "<<EigenValue<<" % \t\t "<<sum<<" %\n";
		//ss <<EigenValue<<"\t\t "<<sum<<"\n";      //<- use this formatting when writing to file 

	}
	return ss.str();
}

// sets the content string within the error text box
void setErrorTextBoxContent()
{	
	std::string text = getErrorText();
	ErrorTextBox->set_text(text.c_str());
}

// instead of setting the text box we can also write to file
void writeErrorToFile(std::string nof)
{
	std::ofstream file(nof.c_str(), std::ios::out | std::ios::app );
	if(file.is_open())
	{
		std::string text = getErrorText();
		file<<"Nr of basis states: "<<myRenderManager->getPCAComponent()->getNrOfBasisStates()<<"\n"<<text.c_str()<<"\n\n";
		//file<<text.c_str()<<"\n";
		file.close();
	}
}

// write out the variance list to a file
void writeVarianceToFile(std::string nof)
{
	
	std::ofstream file(nof.c_str(), std::ios::out | std::ios::app );
	if(file.is_open())
	{
		std::string text = getVarianceText();
		file<<text.c_str()<<"\n\n";
		file.close();
	}
}

// write out all the amplitudes over time of a specific feature to a file
void writeFeatureVectorToFile(std::string nof, int nr)
{
	
	std::ofstream file(nof.c_str(), std::ios::out | std::ios::app );
	if(file.is_open())
	{
		std::vector<float> temp = myRenderManager->getPCAComponent()->getFeatureVector(nr);
		for(unsigned int i = 0; i < temp.size(); i++)
		{
			file<<temp[i]<<"\n";
		}
		file<<"\n\n";
		file.close();
	}

}


// write out min, max and mean amplitudes of a specific feature 
void writeMinMaxMeanFeatureAmplitudeToFile(std::string nof, int nr)
{
	
	std::ofstream file(nof.c_str(), std::ios::out | std::ios::app );
	if(file.is_open())
	{
		float min=0.0f, max=0.0f,mean=0.0f;
		std::vector<float> temp = myRenderManager->getPCAComponent()->getFeatureVector(nr);
		for(unsigned int i = 0; i < temp.size(); i++)
		{
			min = (temp[i]<min) ? temp[i] : min;
			max = (temp[i]>max) ? temp[i] : max;
			mean += temp[i];
		}
		mean /= (float) temp.size();
		file<<min<<"\t\t"<<max<<"\t\t"<<mean<<"\n";
		file.close();
	}

}






/* -------------------  RENDERING   FUNCTIONS  --------------- */

// rebuild textures if dirty
void rebuildTextures(bool useClamping)
{
	myRenderManager->rebuildTextures(DataSetNr, (bool) showReconstructedData, PrincCompNr, (bool) useClamping);
}


// main draw function (GLUT display function)
void draw()
{
	if(SimulationFlag) return;  // simulation provides its own rendering

	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if(myRenderManager->getPCAComponent()->getDimZ() > 1) //3D case
	{
		if(useInterpolationMode)
			myRenderManager->renderMarchCubes(InterpolDataSetNr, IsoLevel, showInfos, PrincCompNr);		
		else		 
			myRenderManager->renderMarchCubes(DataSetNr,IsoLevel);
	}
	else //2D case
	{
		if(useInterpolationMode) //render styles are evaluated in render manager class 
		{
			myRenderManager->renderInterpolatedData(InterpolDataSetNr, showTextures, showMarchingSquaresOverlay, showInfos, showReconstructedData, IsoLevel, PrincCompNr, useRenderClamping);
			return;
		}
		if(showTextures)
			myRenderManager->renderTextures();

		if(showMarchingSquaresOverlay)
			myRenderManager->renderMarchSquares(DataSetNr, IsoLevel);
	}

	if(!useInterpolationMode && showInfos)
		myRenderManager->showInfos((bool) showReconstructedData, DataSetNr, IsoLevel, PrincCompNr);


	glutSwapBuffers();


	//update feature vector visualization if necessarry
	if (showFeatures)
	{
		glutSetWindow(FeatureVectorWindow);
		glutPostRedisplay();
		glutSetWindow(MainWindow);
	}
	

	
	
}

// redraw frame if dirty 
void redraw()
{
	glutSetWindow(MainWindow);
	rebuildTextures((bool) useRenderClamping); // update textures for 2D rendering
	myRenderManager->setDirtyFlag(); // dirty flag is used for 3D rendering (to trigger march cubes algorithm)
	if(showErrorWindow)	setErrorTextBoxContent(); 
	glutPostRedisplay();
}

// draw time evolution of chosen feature vector
void drawFeatureVector()
{
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	myRenderManager->renderFeatureVector(FeatureNr,DataSetNr);
	glutSwapBuffers();
}

/* -------------------------------- EVENTHANDLING -------------------- */

// one frame back
void prevFrame()
{
	
	if(myRenderManager->getPCAComponent()->getNrOfDataSets() != 0)
	{
			if(useInterpolationMode)
			{
				InterpolDataSetNr -= timeStep;
				if(InterpolDataSetNr < 0.0f) // we have reached the first frame -> jump to last frame -1 (so we can interpolate [last-1,last])
					InterpolDataSetNr = (float) (myRenderManager->getPCAComponent()->getNrOfDataSets() - 2);  
				CurDataSetSpinner->set_int_val((int) InterpolDataSetNr); // in case of interpolation set the spinner to lower value
			}
			else
			{
				DataSetNr = (DataSetNr + myRenderManager->getPCAComponent()->getNrOfDataSets() - 1)%myRenderManager->getPCAComponent()->getNrOfDataSets(); //use modulo to jump back to last frame, use "+ myRenderManager->getPCAComponent()->getNrOfDataSets() - 1" to avoid negative values
				CurDataSetSpinner->set_int_val(DataSetNr); // update spinner
			}
			redraw();
	}

}

// one frame forward
void nextFrame()
{
	if(myRenderManager->getPCAComponent()->getNrOfDataSets() != 0)
	{
			if(useInterpolationMode)
			{
				InterpolDataSetNr += timeStep;
				if(InterpolDataSetNr > (float) (myRenderManager->getPCAComponent()->getNrOfDataSets()-2)) // we have reached the last frame -> jump to first frame
					InterpolDataSetNr = 0.0f;
				CurDataSetSpinner->set_int_val((int) InterpolDataSetNr); // in case of interpolation set the spinner to lower value
			}
			else
			{
				DataSetNr = (DataSetNr + 1)%myRenderManager->getPCAComponent()->getNrOfDataSets(); //use modulo to jump back to first frame
				CurDataSetSpinner->set_int_val(DataSetNr); // update spinner
			}
			redraw();
	}

}

// set the standard values for gui elements
void initializeControls()
{
	DataSetNr = 0;
	InterpolDataSetNr = 0.0f;
	PrincCompNr = 1;
	FeatureNr = 1;
	PrincCompForRec = myRenderManager->getPCAComponent()->getNrOfBasisStates();
	upperPC = 1;
	lowerPC = 0;
	showReconstructedData = 1;
	showErrorWindow = 0;
	

	//After loading file we update the controls with new props
	NrOfPCSpinner->set_int_limits(1,myRenderManager->getPCAComponent()->getNrOfDataSets(),GLUI_LIMIT_CLAMP);
	NrOfPCSpinner->set_int_val(PrincCompForRec);


	LowerBoundPCSpinner->set_int_limits(0,upperPC-1,GLUI_LIMIT_CLAMP);
	LowerBoundPCSpinner->set_int_val(lowerPC);
    UpperBoundPCSpinner->set_int_limits(lowerPC+1,myRenderManager->getPCAComponent()->getNrOfPrincipalComponents()-1,GLUI_LIMIT_CLAMP);
	UpperBoundPCSpinner->set_int_val(upperPC);

	PCSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfPrincipalComponents()-1,GLUI_LIMIT_CLAMP);
	PCSpinner->set_int_val(PrincCompNr);

	CurDataSetSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfDataSets()-1,GLUI_LIMIT_WRAP);
	CurDataSetSpinner->set_int_val(DataSetNr);

	FeatureSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfBasisStates()-1,GLUI_LIMIT_CLAMP);
	FeatureSpinner->set_int_val(FeatureNr);

	std::stringstream ss;
	ss << "Total count of data sets:" << myRenderManager->getPCAComponent()->getNrOfDataSets();

	TotalNrOfDataSets->set_text(ss.str().c_str());

	//Update Display
	GLUI_Master.sync_live_all();
	myRenderManager->regenerateTextureIDs();
	redraw();

	
}

// callbacks for buttons in category file
void FileFunction_CB(int ID)
{	
	std::string nof; //name of (imported) file
	pca::PCAfluid *PCA = new pca::PCAfluid(); // generate new pca component (needed in case of loading)

	switch(ID)
	{
		case LOAD:   // Load a PCA state
						
			// pca component provides loading interface itself
			PCA->load();
			// we clear the old pca component...
			myRenderManager->getPCAComponent()->~PCAfluid();
			//... and set up the new one
			myRenderManager->setPCAComponent(PCA);	
			
			initializeControls(); // initialize everything
			//writeVarianceToFile(VARIANCEFILE); //write variance list to file 
			break;


		case SAVE:   // save PCA state

			PCA->~PCAfluid(); // we don't use the new pca component anymore
			myRenderManager->getPCAComponent()->save();
			
			break;

		case IMPORT:

			PCA->~PCAfluid(); // we don't use the new pca component here

			// create gui for import window
			ImportWindow = GLUI_Master.create_glui("Import Data Sets");
			ImportFileBrowser = new GLUI_FileBrowser(ImportWindow,"",false,FILEBROWSERCLICK,FileFunction_CB);
			ImportFileBrowser->set_w(400);

			ImportRadioGroup = ImportWindow->add_radiogroup(&ImportRadioSelection); // bind choice directly to global variable
			
			ImportWindow->add_radiobutton_to_group(ImportRadioGroup, "2D Data");
			ImportWindow->add_radiobutton_to_group(ImportRadioGroup, "3D Data");

			ImportWindow->add_checkbox("Transpose Data Vector",&useTransposing);
			ImportWindow->add_checkbox("Normalize Data Sets",&useNormalizing);


			ImportWindow->add_checkbox("Use Prefiltering",&usePrefiltering);
			PrefilterSpinner = ImportWindow->add_spinner("Threshold value",GLUI_SPINNER_FLOAT,&PrefilterThreshold);
			PrefilterSpinner->set_float_limits(0.0f,1000.0f,GLUI_LIMIT_CLAMP);
			PrefilterSpinner->set_float_val(0.01);
			

			
			ImportWindow->add_button("Cancel", CANCELBUTTON, FileFunction_CB);

			break;

		case FILEBROWSERCLICK: //if we have clicked on a grid file
			
			// load the filesequence of grids
			nof = "";
			nof = ImportFileBrowser->get_file();
			std::cout<<nof.c_str()<<" imported..."<<std::endl;
			
			
			fileSequence = util::FileSequence(nof);		
		   	currentFile = fileSequence.begin();
			

			if (ImportRadioSelection == 0) // import a 2D Data Set
			{
				int dimx, dimy;
				getDataExtends(nof.c_str(),dimx,dimy);
				PCA->initialize(dimx,dimy,1,1, (bool) useTransposing);
				// hook all grids into pca modul
				while(currentFile != fileSequence.end())
				{
					std::vector<float> data;
					importGrid2D(*currentFile,&data, (bool) usePrefiltering, PrefilterThreshold);
					PCA->appendDataSet(data);
					++currentFile;
				}
			}



			if (ImportRadioSelection == 1) // import a 3D Data Set
			{
				unsigned int dimx, dimy, dimz;				
				getDataExtends3D((*currentFile).c_str(),dimx,dimy,dimz);					
				PCA->initialize(dimx,dimy,dimz,1,(bool) useTransposing);
				currentFile = fileSequence.begin();

				// hook all grids into pca modul
				while(currentFile != fileSequence.end())
				{
					std::vector<float> data;
					importGrid3d((*currentFile).c_str(),data,dimx,dimy,dimz);
			
					// prefiltering , simple test filtering with constant iso value (should be implemented seperately)
					if(usePrefiltering)
					{
						for(unsigned int i=0; i < data.size(); i++)
						{
						(data[i] > 0.01f)?(data[i] = 1.0f):(data[i] = 0.0f);
						}
					}
					PCA->appendDataSet(data);
					++currentFile;
				}
			}

			// normalize data sets if neccessary
			if(useNormalizing) PCA->normalizeDataSets();

			
			PCA->executePCA();	 //do the PCA			
			myRenderManager->setPCAComponent(PCA);	//register PCA modul
			initializeControls(); // initialize everything
			//writeVarianceToFile(VARIANCEFILE); // write variance list to file

			/*
			//Aquiring data for diagrams: check out MSEs for any number of PCs 
			for(int i = 0; i < myRenderManager->getPCAComponent()->getNrOfDataSets(); i++)
			{
				myRenderManager->getPCAComponent()->setNrOfBasisStates(i);
				writeErrorToFile(ERRORFILE);
			}
			*/


			/* //Aquiring data for diagrams: check out min/max/mean amplitudes of feature vectors
			myRenderManager->getPCAComponent()->setNrOfBasisStates(myRenderManager->getPCAComponent()->getNrOfDataSets());
			for(int i = 0; i < myRenderManager->getPCAComponent()->getNrOfBasisStates(); i++)
			{
				writeMinMaxMeanFeatureAmplitudeToFile(MINMAXMEANFEATUREAMPLITUDE,i);
			}*/


			
		case CANCELBUTTON:
			ImportWindow->close();
			glutSetWindow(MainWindow);
			break;	
	}




	


}

// HOTKEYS
void keyboardFunction(unsigned char key, int x, int y)   
{
	switch(key)
	{	
		case 'i':
			showInfos = !showInfos;
			break;			
	}
	glutPostRedisplay();
}



// navigate through data sets (can also be done with mouse wheel)
void specialKeysFunction(int key, int x, int y)
{
	switch(key)
	{					
		case GLUT_KEY_LEFT :
			prevFrame();
			break;
		case GLUT_KEY_RIGHT :
			nextFrame();
			break;
	}
	CurDataSetSpinner->set_int_val(DataSetNr);
}


// camera navigation in case of 3D data
void mouseFunction(int button, int state, int x, int y)
{
	//if (glutGetModifiers() == GLUT_ACTIVE_CTRL)                  //middle mouse button only works with modified version of glut.dll (else use CTRL button)
	//{		
	//	if (button == GLUT_LEFT_BUTTON)
	//		cameraState = PANNING;
	//}

		lastMousePosX = x;
		lastMousePosY = y;

		if (button == GLUT_LEFT_BUTTON)
			cameraState = ROTATING;
		
		if (button == GLUT_MIDDLE_BUTTON)
			cameraState = PANNING;

		if (button == GLUT_RIGHT_BUTTON)
			cameraState = ZOOMING;
	
	
	if (state == GLUT_UP )
	{
		cameraState = INACTIVE;
	}
	
	if ( button == GLUT_WHEEL_UP )
		nextFrame();	
	
	if( button == GLUT_WHEEL_DOWN )
		prevFrame();
}



void mouseMotionFunction(int xPos, int yPos)
{	
	
	
	if(cameraState == PANNING || cameraState == ZOOMING || cameraState == ROTATING)
	{

		if(cameraState == PANNING)
		{
			float x = xPos - lastMousePosX;
			float y = yPos - lastMousePosY;
			myRenderManager->viewManipulator.panView( x, -y );			
		}
		
		if(cameraState == ZOOMING)
		{
			float x = xPos - lastMousePosX;
			myRenderManager->viewManipulator.zoomView( -x*myRenderManager->viewManipulator.getDistance()*0.005 );
		}

		if(cameraState == ROTATING)
		{
			myRenderManager->viewManipulator.orbitView( xPos - lastMousePosX, yPos - lastMousePosY );
		}
				

		lastMousePosX = xPos;
		lastMousePosY = yPos;
	}
	glutPostRedisplay();
}


// callback for reshaping the window
void reshapeFunction(int width, int height)
{
	WindowWidth = width;
	WindowHeight = height;
	glViewport(0,0,width,height);
	myRenderManager->setWindowSize(width,height);
}


// callbacks for menu elements
void Controls_CB(int ID)
{
	std::stringstream ss;

	switch(ID)
	{	
		case SHOWFEATCHECKBOX: // this ceckbox triggers a window pop up
			
			if(showFeatures) // pop up the new window to show particular feature vector
			{

				std::stringstream ss;
				ss << "Time Evolution for Feature " << FeatureNr;
				glutInitWindowSize(lastFVWSizeX,lastFVWSizeY);
				glutInitWindowPosition(lastFVWPosX,lastFVWPosY);
				FeatureVectorWindow = glutCreateWindow(ss.str().c_str());
				glutDisplayFunc(drawFeatureVector);
				glutSpecialFunc(specialKeysFunction);
				glutSetWindow(FeatureVectorWindow);
				glutPostRedisplay();
				glutSetWindow(MainWindow);
				//writeFeatureVectorToFile(FEATUREVECTORFILE,FeatureNr); //write feature vector to file
			}

			else    //close window if no features should be shown (we save the window position for the next call)
			{
				glutSetWindow(FeatureVectorWindow);
				lastFVWPosX = glutGet(GLUT_WINDOW_X);
				lastFVWPosY = glutGet(GLUT_WINDOW_Y);
				lastFVWSizeX = glutGet(GLUT_WINDOW_WIDTH);
				lastFVWSizeY = glutGet(GLUT_WINDOW_HEIGHT);
				glutDestroyWindow(FeatureVectorWindow);
				glutSetWindow(MainWindow);
			}
			break;

		case APPLYREC: // button to apply changes in number of PCs

			myRenderManager->getPCAComponent()->setNrOfBasisStates(PrincCompForRec); // we set the new number of basis states
			FeatureSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfBasisStates()-1,GLUI_LIMIT_CLAMP); // also set the number of reachable feature vectors
			//writeErrorToFile(ERRORFILE); //in case of gathering some data for diagrams
			break;


		case APPLYBOUND: // apply the changes for PC interval

			myRenderManager->getPCAComponent()->setBasisStatesInterval(lowerPC,upperPC); // set interval with lower and upper boundary
			FeatureSpinner->set_int_limits(0,upperPC-lowerPC-1,GLUI_LIMIT_CLAMP); // also set the number of reachable feature vectors
			//writeErrorToFile(ERRORFILE); //in case of gathering some data for diagrams
			break;

		case CURDATASETSPINNER: // gui element providing the current data set number

			InterpolDataSetNr = DataSetNr;
			break;

		case CURFEATURE:  // gui spinner where we can choose the feature vector shown in separate window

			glutSetWindow(FeatureVectorWindow);		
			ss << "Time Evolution for Feature " << FeatureNr;
			glutSetWindowTitle(ss.str().c_str());
			glutPostRedisplay();
			glutSetWindow(MainWindow);
			break;

		case RECONSTRUCTIONRADIOGROUP:

			if(RadioButtonSelection == 0)     // reconstruction with upper PC boundary 
			{
				showReconstructedData = 1;
				PCSpinner->disable();
				NrOfPCSpinner->enable();
				LowerBoundPCSpinner->disable();
				UpperBoundPCSpinner->disable();

				ApplyButton->enable();
				ApplyBoundButton->disable();
			}

			if(RadioButtonSelection == 1)    // reconstruction with lower and upper PC boundary
			{
				showReconstructedData = 1;
				PCSpinner->disable();
				NrOfPCSpinner->disable();			
				LowerBoundPCSpinner->enable();
				UpperBoundPCSpinner->enable();

				ApplyButton->disable();
				ApplyBoundButton->enable();
			}

			if(RadioButtonSelection == 2)    // show the PC itself
			{
				if(!myRenderManager->getPCAComponent()->useTransposing()) showReconstructedData = 0;
				NrOfPCSpinner->disable();
				PCSpinner->enable();
				LowerBoundPCSpinner->disable();
				UpperBoundPCSpinner->disable();

				ApplyButton->disable();
				ApplyBoundButton->disable();
			}
				
			break;

		case PCSPINNER: //spinner gui element to choose a certain PC, we can put extra functionality in here
			break;

		case ISOSPINNER:
			redraw(); // we have changed the iso level			
			break;

		case SHOWTEXTURESCHECKBOX: // checkbox gui element to decide if texture shall be shown, we can put extra functionality in here
			break;

		case LOWERPCSPINNER: //spinner gui element to define lower PC boundary
		case UPPERPCSPINNER: //spinner gui element to define upper PC boundary

			LowerBoundPCSpinner->set_int_limits(0,upperPC-1,GLUI_LIMIT_CLAMP);	
			UpperBoundPCSpinner->set_int_limits(lowerPC+1,myRenderManager->getPCAComponent()->getNrOfPrincipalComponents()-1,GLUI_LIMIT_CLAMP);
			break;


		case SHOWEIGENVALUESCHECKBOX: 
			
			if(showEigenvalues) // pop up a new window to show eigenvalues
			{
				EigenvalueWindow = GLUI_Master.create_glui("Eigenvalues");
				EigenvalueWindow->set_main_gfx_window(MainWindow);

				//create content
				std::string text = getVarianceText();
				GLUI_TextBox *TextBox = new GLUI_TextBox(EigenvalueWindow,true);
				TextBox->set_w(300);
				TextBox->set_h(600);
				TextBox->set_text(text.c_str());
				EigenvalueWindow->add_button("Close",EIGENVALUESCLOSE, Controls_CB);
			}

			else    //close window by unchecking
			{
				EigenvalueWindow->close();
				glutSetWindow(MainWindow);
			}
			break;


		case EIGENVALUESCLOSE:  //close window by pressing close button
			EigenvalueWindow->close();
			showEigenvalues = 0; // don't forget boolean variable representing the checkbox state	
			glutSetWindow(MainWindow);
			break;

		case SHOWERRORWINDOWCHECKB: 

			if(showErrorWindow) // pop up a new window to show reconstruction error
			{
				//create window
				ErrorWindow = GLUI_Master.create_glui("Reconstruction Errors");
				ErrorWindow->set_main_gfx_window(MainWindow);
			
				//create content
				ErrorTextBox = new GLUI_TextBox(ErrorWindow);
				ErrorTextBox->set_w(300);
				ErrorTextBox->set_h(100);
				setErrorTextBoxContent();
				ErrorWindow->add_button("Close",CLOSEERRORWINDOWBUTTON, Controls_CB);
			}
			else   //close window by unchecking
			{
				ErrorWindow->close();
				glutSetWindow(MainWindow);
			}
			break;

		case CLOSEERRORWINDOWBUTTON: //close window by pressing close button
			ErrorWindow->close();
			showErrorWindow = 0; // don't forget boolean variable representing the checkbox state	
			glutSetWindow(MainWindow);
			break;

		case TIMESTEPCHECKBOX: // checkbox gui element to activate data set interpolation, we have to mutually update values for DataSetNr and InterpolDataSetNr to keep them updated
			if(useInterpolationMode)
				InterpolDataSetNr = (float) DataSetNr;
			else
				DataSetNr = (int) InterpolDataSetNr;
			break;

		case TIMESTEPSPINNER: // we have to watch out that there is no negative time step
			if(timeStep <= 0.0f) timeStep = 0.0f;				
			break;

	}
	GLUI_Master.sync_live_all(); // sync all the gui elements and redraw interface
	redraw();	
}



void animate() // if animation is activated we have to go to the next frame
{
	nextFrame();	
}


void idleFunction() // GLUT idle function, checks if simulation or animation is activated
{
	glutSetWindow(MainWindow);
	if(SimulationFlag)
	{
		simulateStep();
	}
	else if(animateFlag)
	{
		animate();	
	}
}







// ---------------------------  MAIN FUNCTION ------------------------------------------------
int main( int argc, char **argv )
{
	pca::PCAfluid *PCA = new pca::PCAfluid();	// create main PCA modul, initialize and register at render manager
	PCA->initialize(0,0,0,1,false);	
	myRenderManager->setPCAComponent(PCA);
	

	//set oscillators with some predefined values
	Oscillator *osc1 = new Oscillator(1, 0.0f, 0.1f, 30.0f, 0.0f, 1.0f, 0.0f, 0);
	Oscillator *osc2 = new Oscillator(1, 0.0f, 0.05f, 20.0f, 1.0f, 1.0f, 0.0f, 1); 
	Oscillator *osc3 = new Oscillator(1, 0.0f, 0.05f, 10.0f, -1.0f, 1.0f, 0.0f, 3); 
	OscillatorList.push_back(osc1);
	OscillatorList.push_back(osc2);
	OscillatorList.push_back(osc3);
		

	//----------- Set up GLUT specific initializations and callback functions
	//glutInit(&argc, argv );
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	MainWindow = glutCreateWindow("PCAfluid");
	glutReshapeWindow(WindowWidth,WindowHeight);
	//glutShowWindow();
	
	

	glutDisplayFunc(draw);
	//glutIdleFunc(GLUI_Master);
	GLUI_Master.set_glutIdleFunc(idleFunction);
	glutKeyboardFunc(keyboardFunction);
	glutSpecialFunc(specialKeysFunction);
	glutMouseFunc(mouseFunction);
	glutMotionFunc(mouseMotionFunction);
	glutReshapeFunc(reshapeFunction);



	// ---------------------------- GLUI Main Controls Layout Definitions	-------------------------------------------------- 

	Controls = GLUI_Master.create_glui("Main Controls"); // main controls window
	Controls->set_main_gfx_window(MainWindow);	
	GLUI_Panel *TopPanel = Controls->add_panel("",GLUI_PANEL_NONE);
	TopPanel->set_alignment(GLUI_ALIGN_LEFT);



	//File functions: load, save, import
	GLUI_Panel *FilePanel = Controls->add_panel_to_panel(TopPanel,"FILE OPERATIONS");
	Controls->add_button_to_panel(FilePanel, "Open...", LOAD, FileFunction_CB);
	Controls->add_column_to_panel(FilePanel,0);
	Controls->add_button_to_panel(FilePanel, "Import...",  IMPORT, FileFunction_CB);
	Controls->add_column_to_panel(FilePanel,0);
	Controls->add_button_to_panel(FilePanel, "Save...", SAVE, FileFunction_CB);
	Controls->add_column_to_panel(TopPanel,0);
	

	// Rollouts for every category
	GLUI_Rollout *AnalysisInfosRollOut = Controls->add_rollout("Analysis Informations",0);
	AnalysisInfosRollOut->set_alignment(GLUI_ALIGN_LEFT);
	AnalysisInfosRollOut->set_w(220);

	GLUI_Rollout *ReconstructionRollOut = Controls->add_rollout("Reconstruction");
	ReconstructionRollOut->set_alignment(GLUI_ALIGN_LEFT);	

	GLUI_Rollout *RenderingRollOut = Controls->add_rollout("Rendering");
	RenderingRollOut->set_alignment(GLUI_ALIGN_LEFT);
	
	GLUI_Rollout *SimulationRollOut = Controls->add_rollout("Simulation",0);
	SimulationRollOut->set_alignment(GLUI_ALIGN_LEFT);
	SimulationRollOut->set_w(448);
	

	// ----------------------- Category ANALYSIS INFOS ------------------------------------------------------
	GLUI_Panel *InfoPanel = Controls->add_panel_to_panel(AnalysisInfosRollOut,"",GLUI_PANEL_NONE);
	InfoPanel->set_alignment(GLUI_ALIGN_LEFT);
	
	// total number of data sets
	std::stringstream ss;
	ss << "Total number of data sets: " << myRenderManager->getPCAComponent()->getNrOfDataSets();
	TotalNrOfDataSets = Controls->add_statictext_to_panel(InfoPanel,ss.str().c_str());

	// current data set
	CurDataSetSpinner = Controls->add_spinner_to_panel(InfoPanel,"Current data set:", GLUI_SPINNER_INT, &DataSetNr, CURDATASETSPINNER, Controls_CB);
	CurDataSetSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfDataSets()-1,GLUI_LIMIT_WRAP);
	
	// Reconstruction Error Window
	Controls->add_checkbox_to_panel(InfoPanel,"Show reconstruction error", &showErrorWindow, SHOWERRORWINDOWCHECKB, Controls_CB);	
	// show eigen values
	Controls->add_checkbox_to_panel(InfoPanel,"Show list of eigenvalues", &showEigenvalues, SHOWEIGENVALUESCHECKBOX, Controls_CB);
	// Feature vector visualization
	Controls->add_checkbox_to_panel(InfoPanel, "Show time evolution of features", &showFeatures, SHOWFEATCHECKBOX, Controls_CB);
	FeatureSpinner = Controls->add_spinner_to_panel(InfoPanel, "Nr. of feature:", GLUI_SPINNER_INT,&FeatureNr,CURFEATURE, Controls_CB);
	FeatureSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfBasisStates()-1,GLUI_LIMIT_CLAMP);
	

	//-------------- Category RECONSTRUCTION -------------------------------------
	//upper part
	GLUI_Panel *ReconstructionPanel = Controls->add_panel_to_panel(ReconstructionRollOut,"",GLUI_PANEL_NONE);
	ReconstructionPanel->set_alignment(GLUI_ALIGN_LEFT);
	ReconstructionPanel->set_h(500);
    // choose reconstruction method or PC visualization
	ReconstructionGroup = Controls->add_radiogroup_to_panel(ReconstructionPanel, &RadioButtonSelection, RECONSTRUCTIONRADIOGROUP,Controls_CB);
	Controls->add_radiobutton_to_group(ReconstructionGroup, "Show reconstructed data sets");
	Controls->add_radiobutton_to_group(ReconstructionGroup, "Show reconstruction using upper/lower PC bounds");
	Controls->add_radiobutton_to_group(ReconstructionGroup, "Show Principal Components");

	Controls->add_separator_to_panel(ReconstructionPanel);
	
	//lower part
	GLUI_Panel *RecSettingsPanel = Controls->add_panel_to_panel(ReconstructionPanel, "",GLUI_PANEL_NONE);
	RecSettingsPanel->set_alignment(GLUI_ALIGN_LEFT);

	// gui elements for simple reconstruction
	NrOfPCSpinner = Controls->add_spinner_to_panel(RecSettingsPanel,"Nr. of PCs:",GLUI_SPINNER_INT, &PrincCompForRec);
	NrOfPCSpinner->set_int_limits(1,myRenderManager->getPCAComponent()->getNrOfDataSets(),GLUI_LIMIT_CLAMP);
	Controls->add_statictext_to_panel(RecSettingsPanel,"")->set_h(20); //placeholder
	ApplyButton = Controls->add_button_to_panel(RecSettingsPanel,"Apply", APPLYREC, Controls_CB);
	ApplyButton->set_alignment(GLUI_ALIGN_LEFT);

	Controls->add_column_to_panel(RecSettingsPanel,0);
	
	// gui elements for reconstruction with lower/upper boundaries
	LowerBoundPCSpinner = Controls->add_spinner_to_panel(RecSettingsPanel,"lower:",GLUI_SPINNER_INT, &lowerPC,LOWERPCSPINNER,Controls_CB);
	LowerBoundPCSpinner->set_int_limits(0,upperPC-1,GLUI_LIMIT_CLAMP);
	LowerBoundPCSpinner->disable();
	UpperBoundPCSpinner = Controls->add_spinner_to_panel(RecSettingsPanel,"upper:",GLUI_SPINNER_INT, &upperPC, UPPERPCSPINNER,Controls_CB);
	UpperBoundPCSpinner->set_int_limits(lowerPC+1,myRenderManager->getPCAComponent()->getNrOfPrincipalComponents()-1,GLUI_LIMIT_CLAMP);
	UpperBoundPCSpinner->disable();
	ApplyBoundButton = Controls->add_button_to_panel(RecSettingsPanel,"Apply", APPLYBOUND, Controls_CB);
	ApplyBoundButton->disable();

	Controls->add_column_to_panel(RecSettingsPanel,0);
		
	// gui elements for PC visualization
	PCSpinner = Controls->add_spinner_to_panel(RecSettingsPanel,"PC:",GLUI_SPINNER_INT, &PrincCompNr, PCSPINNER, Controls_CB);
	PCSpinner->set_int_limits(0,myRenderManager->getPCAComponent()->getNrOfPrincipalComponents()-1,GLUI_LIMIT_CLAMP);
	PCSpinner->disable();
	


	// -------------------------------- Category RENDERING --------------------------------------------------
	GLUI_Panel *RenderingPanel = Controls->add_panel_to_panel(RenderingRollOut,"",GLUI_PANEL_NONE);
	RenderingPanel->set_alignment(GLUI_ALIGN_LEFT);

	// Textures
	Controls->add_checkbox_to_panel(RenderingPanel,"Show Textures", &showTextures, SHOWTEXTURESCHECKBOX, Controls_CB);

	// Use Clamping Renderer
	Controls->add_checkbox_to_panel(RenderingPanel,"Render textures with clamped values", &useRenderClamping);

	// Isolevel	
	Controls->add_checkbox_to_panel(RenderingPanel, "Show isolevel", &showMarchingSquaresOverlay);
	Controls->add_spinner_to_panel(RenderingPanel, "Iso value:", GLUI_SPINNER_FLOAT, &IsoLevel, ISOSPINNER,Controls_CB);
	
	Controls->add_column_to_panel(RenderingPanel,0);

	// Animation
	GLUI_Panel *AnimationPanel = Controls->add_panel_to_panel(RenderingPanel,"Animation/Interpolation");
	Controls->add_checkbox_to_panel(AnimationPanel,"Animate data sets", &animateFlag);
	Controls->add_checkbox_to_panel(AnimationPanel,"Interpolate data sets", &useInterpolationMode, TIMESTEPCHECKBOX, Controls_CB);
	Controls->add_spinner_to_panel(AnimationPanel,"Time step:",GLUI_SPINNER_FLOAT, &timeStep, TIMESTEPSPINNER, Controls_CB);

	//-------------------------------- Category SIMULATION --------------------------------------------------

	GLUI_Panel *SimuPanel = Controls->add_panel_to_panel(SimulationRollOut,"",GLUI_PANEL_NONE);
	
    // Simulation time controls (time step, start/stop, reset etc.)
	SimuTimeText = Controls->add_edittext_to_panel(SimuPanel,"Simulated Time:");
	SimuTimeText->set_float_val(0.0f);
	Controls->add_spinner_to_panel(SimuPanel,"Time step size:",GLUI_SPINNER_FLOAT,&SimuTimeStepSize,SIMUTIMESTEPSPINNER,SimulationControl);
	Controls->add_button_to_panel(SimuPanel,"Start/Pause",STARTSIMUBUTTON,SimulationControl);
	Controls->add_button_to_panel(SimuPanel,"Stop/Reset",RESETSIMUBUTTON,SimulationControl);
	Controls->add_button_to_panel(SimuPanel,"Add Oscillator",ADDOSCILLATORBUTTON,SimulationControl);

	// GUIs for every oscillator in separate function 
	Controls->add_column_to_panel(SimuPanel);
	OscillatorPanel = Controls->add_panel_to_panel(SimuPanel,"",GLUI_PANEL_NONE);
	
	for(unsigned int i=0; i<OscillatorList.size(); i++)
	{
		createOscillatorGUI(OscillatorList[i],getSimuID());		
	}
	glutMainLoop();
	

}



// ------------------- SIMULATION ZONE ------------------------------------------------------------- 


void removeOscillator(int ID) // remove Oscillator and gui
{
	for (unsigned i=0; i<OscillatorList.size();i++)
	{
		if(OscillatorList[i]->_ID == ID)
		{
			OscillatorList[i]->_GUIPtr->unlink();
			//delete OscillatorList[i]._GUIPtr;
			OscillatorList.erase(OscillatorList.begin()+i);
		}		
	}

	

}

void createOscillatorGUI(Oscillator *osc, int ID) // create oscillator gui with all neccessary elements
{	
	std::stringstream ss;
	ss<<"Oscillator "<<ID+1;
	GLUI_Rollout *Rollout = Controls->add_rollout_to_panel(OscillatorPanel,ss.str().c_str(),0);
	Rollout->set_w(240);
	GLUI_Panel *OscPanel = Controls->add_panel_to_panel(Rollout,"",GLUI_PANEL_NONE);	
		
	GLUI_Spinner *NrOfPC = Controls->add_spinner_to_panel(OscPanel,"                    Nr. of PC:",GLUI_SPINNER_INT,&(osc->_NrOfPC));
	NrOfPC->set_int_limits(0,1000);

	Controls->add_spinner_to_panel(OscPanel,"Fundamental frequency:",GLUI_SPINNER_FLOAT,&(osc->_FundamentalFrequency));

	GLUI_Spinner *NrOfHarmonicsSpinner = Controls->add_spinner_to_panel(OscPanel,"          Nr. of harmonics:",GLUI_SPINNER_INT,&(osc->_NrOfHarmonics));
	NrOfHarmonicsSpinner->set_int_limits(1,1000);

	Controls->add_spinner_to_panel(OscPanel,"      Harmonic Damping:",GLUI_SPINNER_FLOAT,&(osc->_HarmonicDamping));
	Controls->add_spinner_to_panel(OscPanel,"                        Phase:",GLUI_SPINNER_FLOAT,&(osc->_Phase));
	Controls->add_spinner_to_panel(OscPanel,"                   Amplitude:",GLUI_SPINNER_FLOAT,&(osc->_Amplitude));
	Controls->add_spinner_to_panel(OscPanel,"     Amplitude Damping:",GLUI_SPINNER_FLOAT,&(osc->_AmplitudeDamping));
	Controls->add_spinner_to_panel(OscPanel,"                            DC:",GLUI_SPINNER_FLOAT,&(osc->_DC));
	Controls->add_button_to_panel(OscPanel,"Remove",ID,removeOscillator);

	osc->_GUIPtr = Rollout;
	osc->_ID = ID;
}


void SimulationControl(int ID) // event handling for all simulation related gui elements
{
	Oscillator *osc = new Oscillator();

	switch(ID)
	{
		case RESETSIMUBUTTON:  // reset the simulation
			SimulationFlag = 0;
			SimuTime = 0.0f;
			resetOscillators();
			{
			std::stringstream ss;
			ss<<SimuTime;
			SimuTimeText->set_text(ss.str().c_str());
			}
			GLUI_Master.sync_live_all();
			break;

		case STARTSIMUBUTTON: // start the simulation
			SimulationFlag = !SimulationFlag;	
			break;
		
		case SIMUTIMESTEPSPINNER: // set current time step size to every oscillator in list
			for(unsigned int i=0; i<OscillatorList.size(); i++)
			{
				OscillatorList[i]->_StepSize = SimuTimeStepSize;						
			}
			break;

		case ADDOSCILLATORBUTTON: // add a new oscillator
			OscillatorList.push_back(osc);	//push new oscillator into list	
			osc->_StepSize = SimuTimeStepSize;	//sync the oscillators time step size with global simulation step size				
			createOscillatorGUI(osc,getSimuID()); //create GUI
						
			break;
			



	}
}	

void resetOscillators() // resets all oscillators
{
	for(unsigned int i=0; i<OscillatorList.size(); i++)
			{
				OscillatorList[i]->_Time = SimuTime;
				OscillatorList[i]->_StepSize = SimuTimeStepSize;				
			}	

}


void simulateStep() // one step forward in time
{	
	SimuTime += SimuTimeStepSize;
	std::stringstream ss;
	ss<<SimuTime;
	SimuTimeText->set_text(ss.str().c_str());
	
		

	for (unsigned int i = 0; i < OscillatorList.size(); i++) 	//let all oscillators do one step
	{
		OscillatorList[i]->step();				
	}
		

	std::vector<float> features;
	for (unsigned int i = 0; i < OscillatorList.size(); i++)
	{
		features.push_back(OscillatorList[i]->getValue());  
	}


	//construct data vector from oscillator values as features
	std::vector<float> data;
	std::vector<float> MeanValues = myRenderManager->getPCAComponent()->getMeanValues();
	data.resize(myRenderManager->getPCAComponent()->getDataSetSize(), 0.0f);
	for(unsigned i=0; i<features.size(); i++)
	{
		std::vector<float> temp = myRenderManager->getPCAComponent()->getPrincipalComponent(OscillatorList[i]->_NrOfPC);
		for(unsigned j=0; j<data.size(); j++)
		{
			data[j] += features[i] * temp[j];
			if(i==0) 
				data[j] += MeanValues[j];   //<- don't forget to add the corresponding mean values (PCs are computed from mean free data), but only once;
		}
	}
	
	// Render data vector
	if(myRenderManager->getPCAComponent()->getDimZ() > 1) //3D case
	{
		myRenderManager->setDirtyFlag();
		myRenderManager->renderMarchCubes(data,data,IsoLevel);
		glutSwapBuffers();	
	}

	else // 2D case
	{
		glClearColor(1.0, 1.0, 1.0, 1.0);
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		myRenderManager->buildTextureFromDataVector(myRenderManager->getPCAComponent()->getDimX(),myRenderManager->getPCAComponent()->getDimY(), data, data, useRenderClamping);
		if(showTextures)
			myRenderManager->renderTextures();
		if(showMarchingSquaresOverlay)
			myRenderManager->renderMarchSquares(data, data, IsoLevel);
		glutSwapBuffers();		
	}
	
	
	



}




